#
# $Id: Subscriber.py 54384 2007-11-24 18:35:03Z naro $
#

"""
Subscriber main class
"""

# Standard Python imports
import string

# Zope core imports
from Globals import InitializeClass
from AccessControl import Permissions, getSecurityManager, ClassSecurityInfo, Unauthorized
from AccessControl.SecurityManagement import newSecurityManager

# CMF/Plone imports
from Products.CMFCore.permissions import View, ManageProperties, ListFolderContents
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName

# Product specific imports
from PNLPermissions import *
from PNLUtils import ownerOfObject
from PNLUtils import checkMailAddress
from PNLBase import PNLContentBase

#################
## The factory ##
#################

def addSubscriber(self, id, title = '', REQUEST = {}):
    """
    Factory method for a Subscriber object
    """
    obj = Subscriber(id, title)
    self._setObject(id, obj)
    getattr(self, id)._post_init()
    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main')

#################################
## The Subscriber content type ##
#################################

class Subscriber(PortalContent, DefaultDublinCoreImpl, PNLContentBase):
    """Subscriber class"""
    
    ########################################
    ## Registration info for portal_types ##
    ########################################

    factory_type_information = {
        'id': 'Subscriber',
        'portal_type': 'Subscriber',
        'meta_type': 'Subscriber',
        'description': 'A newletter subscriber (has no sense oudside a NewsletterTheme object)',
        'content_icon': 'Subscriber.gif',
        'product': 'PloneGazette',
        'factory': 'addSubscriber',
        'immediate_view': 'Subscriber_edit',
        'global_allow': 0,
        'filter_content_types': 0,
        'allowed_content_types': (),
        'actions': (
            {
                'id': 'view',
                'name': 'View',
                'action': 'string:${object_url}/Subscriber_view',
                'permissions': (View,),
                'category': 'object'
                },
            {
                'id': 'edit',
                'name': 'Edit',
                'action': 'string:${object_url}/Subscriber_editForm',
                'permissions': (ChangeSubscriber,),
                'category': 'object',
                },
            ),
        'aliases' : {
                'edit'       : 'base_edit',
            }, 
    }
    
    ###########################
    ## Basic class behaviour ##
    ###########################

    __implements__ = PortalContent.__implements__

    meta_type = factory_type_information['meta_type']

    manage_options = PortalContent.manage_options

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    # Init method
    security.declarePrivate('__init__')
    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""

        # Subscriber.inheritedAttribute('__init__')(self,)
        self._internalVersion = 1
        self.id = id
        self.title = title
        self.format = 'HTML'
        self.active = False
        return

    security.declarePrivate('_post_init')
    def _post_init(self):
        """
        _post_init(self) => Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """

        # self.indexObject()
        return

    #############################
    ## Content editing methods ##
    #############################

    # Edit method (change this to suit your needs)
    # This edit method should only change attributes that are neither 'id' or metadatas.
    security.declareProtected(ChangeSubscriber, 'edit')
    def edit(self, format='', active=False, title=''):
        """
        edit(self, text = '') => object modification method
        """

        # Change attributes
        self.format = format
        self.active = active
        self.title = title.strip()

        # Reindex
        # self.reindexObject()
        return

    security.declarePublic('activateOnFirstTime')
    def activateOnFirstTime(self, REQUEST):
        """Activation for first time access
        """
        firsttime = REQUEST.form.get('firsttime')
        if firsttime:
#            newSecurityManager(REQUEST, ownerOfObject(self.getTheme()))
            self.active = True
        return


    def checkMailAddress(self, mail):
        return checkMailAddress(self,mail)
    
    #####################
    ## Utility methods ##
    #####################

    security.declareProtected(ChangeSubscriber, 'mailingInfo')
    def mailingInfo(self):
        """returns None or (email, format, edit_url) tuple."""
        
        if self.active:
            return self.title, self.format, self.absolute_url() + '/Subscriber_editForm'
        return

    ############################
    ## Hooks for base classes ##
    ############################

    # CMFCatalogAware prevent Subscriber (re)indexing

    def indexObject(self):
        """see CMFCore.CMFCatalogAware"""
        return
    def reindexObject(self, idxs=[]):
        """see CMFCore.CMFCatalogAware"""
        return

# Class instanciation
InitializeClass(Subscriber)
